import pandas as pd


def get_operation_model(cv_model: dict, cv_splits: list, X: pd.DataFrame, y_true: pd.Series, metric:str='test_sp', less_is_better:bool=False) -> tuple:
    """"
    get_operation_model(cv_model: dict, cv_splits: list, X: pd.DataFrame, y_true: pd.Series, metric:str='test_sp', less_is_better:bool=False) -> tuple

    Returns the operation model and its corresponding data. For that, it finds
    the model yielding the best metric over its corresponding testing set.

    Input parameters:
      - cv_model: a dictionary with structure equivalent to the one returned by sklearn.model_selection.cross_validate.
      - cv_splits: a list where each item is a tuple containing the indexes to be used for training and testing in
                   see sklearn.model_selection.KFold, for isntance, for details.
      - X: a DataFrame containing **ALL** input samples used for model development and testing. It should be the same X as
           the one passed to sklearn.model_selection.cross_validate.
      - y_true: a Series containing **ALL** target samples used for model development and testing. It should be the same y_true as
           the one passed to sklearn.model_selection.cross_validate.
      - metric: a key within cv_model containing the metric to use to compare cv folds.
      - less_is_better: if True, the best fold will be the one where the passed metric is the lowest (MSE, for instance). The highest
                        will be returned if this value is set to True (Accuracy, for instance).
    
    Returns:
      - best_fold: the id of the fold yielding the operation model.
      - X: the input set used for testing in best_fold.
      - y_true: the target set used for testing in best_fold.
      - model: the operation model yielded by best_fold
    """
    best_fold = cv_model[metric].argmin() if less_is_better else cv_model[metric].argmax()
    _, tst_idx = cv_splits[best_fold]
    X = X.iloc[tst_idx]
    y_true = y_true.iloc[tst_idx]
    model = cv_model['estimator'][best_fold]
    return best_fold, X, y_true, model



import os
import re
import pandas as pd
import tempfile
import mlflow
import lpsds.metrics

def log_dataframe(df: pd.DataFrame, var_name: str, folder: str=''):
    """"
    def log_dataframe(df, var_name, folder='')

    Saves a pandas dataframe to MLFlow as a .parquet file.

    Input parameters:
      - df: the pandas.DataFrame object to be saved.
      - var_name: the name the dataframe will have within MLFlow.
      - folder: the path (in MLFlow) to where the dataframe will be saved.
    """

    with tempfile.TemporaryDirectory() as temp_path:
        temp_file_name = os.path.join(temp_path, var_name + '.parquet')
        df.to_parquet(temp_file_name, index=False)
        mlflow.log_artifact(temp_file_name, folder)


def log_statistics(cv_model: dict) -> dict:
    """
    def log_statistics(cv_model)

    Log metrics obtained via cross validation to MLFlow as statistical summaries.
    I.e.: the mean value obtained across all folds and its minimum and maximum
    values so right value will be found in this range (err_min, mean, err_max)
    with 95% C.I.

    The metrics will be found automatically by looking to keys within cv_model
    object that has the pattern "test_*".

    Input:
        cv_model: the result of the sklearn cross_validate function.
    
    Returns a map where keys are the metric name and values are its mean, err min and err max
    """
    regexp = re.compile(r'test_(.+)')
    ret_map = {}
    for metric, values in cv_model.items():
        grp = regexp.match(metric)
        if grp is not None:
            mean, err_min, err_max = lpsds.metrics.bootstrap_estimate(values)
            metric_name = grp.group(1)
            ret_map[metric_name] = {
                f'{metric_name}_err_min' : err_min,
                f'{metric_name}_mean' : mean,
                f'{metric_name}_err_max' : err_max
            }
            mlflow.log_metrics(ret_map[metric_name])
    
    return ret_map
